#! /usr/bin/env python3
# -*- coding:utf-8 -*-
'''
if_service_exists(): Boolean：服务是否存在
if_service_running(): Booleans:服务是否运行
start_run_service(): 无法以admin运行
'''
import os, subprocess, time
import ctypes, sys
# from tool import bat
import zhangwei_helper.function.Os as self_os
import zhangwei_helper.enum.SelfEnum as self_enum

def if_service_exists(service='MySQL'):
    task = subprocess.Popen('sc query state= all | find /i "DISPLAY_NAME: %s"'
                            % service,
                            stdin=subprocess.PIPE,
                            stdout=subprocess.PIPE, shell=True)
    if len(task.stdout.readlines()) == 0:
        return False
    else:
        return True

def if_service_running(service='MySQL'):
    task = subprocess.Popen('tasklist /nh | find /i "%s"' % service,
                            stdin=subprocess.PIPE,
                            stdout=subprocess.PIPE, shell=True)
    if len(task.stdout.readlines()) == 0:
        return False
    else:
        return True


# def start_run_service(service='MySQL'):
#     '''
#     非admin无法启动service
#     :param service:
#     :return:
#     '''
#     cmd = 'net start %s' % service
#     if self_os.windows_login_as_admin():
#         # 将要运行的代码加到这里
#         # if not win_check_mysql_running():
#         subprocess.Popen(cmd,
#                          stdin=subprocess.PIPE,
#                          stdout=subprocess.PIPE, shell=True)
#     else:
#         if self_os.get_python_major_version() == self_enum.PythonVersion.Python3:
#             # subprocess.Popen("runas /savecred /user:Administrator cmd",
#             #                  shell=True)
#             ctypes.windll.shell32.ShellExecuteW(None, "runas",
#                                                 sys.executable, cmd, None, 1)
#             # time.sleep(10)


# if if_service_exists('mysql'):
#     if not if_service_running('mysql'):
#         start_run_service('mysql')


# -*- coding:utf-8 -*-
'''
网络相关操作
extract_protocol_from_url(url)：获得协议http或者https
extract_host_from_url(url)：获得host
extract_base_url_from_url(url)：获得基础url  https://github.com
gen_proxies_from_ip(ip): 根据IP生成request/request_html需要的代理
detect_if_need_proxy(url): Boolean:是否需要代理
detect_if_proxy_usable(proxies, timeout=5, url='https://www.baidu.com'）：Boolean：代理是否有效
detect_url_exist(url, proxies, headers): url是否存在（返回404）
send_request_get_response(**args): request_html或者error。同步获得页面html
async_send_request_get_response(**args): request_html或者error。异步获得页面html
download_file(url,save_path): Error(下载失败）；None（下载成功）
download_unzip_chrome_driver:
download_unzip_firefox_driver:
'''
from urllib.parse import urlparse
import os

# requests 用来为requests_html提供Error
import requests
from requests_html import HTMLSession
from requests_html import AsyncHTMLSession

import zhangwei_helper.const.Const as self_const
import zhangwei_helper.enum.SelfEnum as self_enum
import zhangwei_helper.function.Os as self_os
import zhangwei_helper.function.Software as self_software


def extract_protocol_from_url(url):
    '''

    :param url:
    :return: http或者https
    '''
    return urlparse(url).scheme


def extract_host_from_url(url):
    '''

    :param url:
    :return: github.com
    '''
    return urlparse(url).netloc


def extract_base_url_from_url(url):
    return urlparse(url).scheme+'://'+urlparse(url).netloc


def gen_proxies_from_ip(ip):
    return {'http':ip,'https':ip}


def detect_if_need_proxy(url):
    try:
        HTMLSession().get(url, headers=self_const.HEADER, timeout=10)
    except requests.exceptions.Timeout as e:
        print('不通过代理发起的请求超时，需要使用代理')
        return True
    except requests.exceptions.ConnectionError as e:
        print('不通过代理发起的请求连接错误，需要使用代理')
        return True
    return False


def detect_if_proxy_usable(proxies, timeout=5, url='https://www.baidu.com'):
    try:
        # ssl._create_default_https_context = ssl._create_unverified_context
        HTMLSession().get(url, headers=self_const.HEADER,
                          proxies=proxies, timeout=timeout)

        # if r.status_code != 200:
        #     return False
    except requests.exceptions.Timeout as e:
        print('代理无效：超时')
        return False
    except requests.exceptions.ProxyError as e:
        print('代理无效：代理错误')
        return False
    except requests.exceptions.ConnectionError as e:
        print('代理无效：连接错误')
        return False
    return True


def detect_url_exist(url, proxies, headers=self_const.HEADER):
    '''
    检测url是否存在（是否返回404）
    :param url:
    :param proxies:
    :param headers:
    :return:
    '''
    r = HTMLSession().get(url, headers=headers, proxies=proxies)
    return r.status_code != 404


def send_request_get_response(url, if_use_proxy=False, proxies=None,
                              header=self_const.HEADER,
                              force_render=False):
    '''
    为了和async_send_request_get_response的参数保持一致，取消force_render
    :param url:
    :param if_use_proxy:  boolean
    :param proxies: dict，如果if_need_proxy为true，传入代理
    :param header: request的header
    :param force_render: 是否要进行render，默认True。如果是静态页面，无需render
    :return:
    '''
    # ssl._create_default_https_context = ssl._create_unverified_context
    if if_use_proxy:
        r = HTMLSession().get(url, headers=header, proxies=proxies,
                              timeout=5)
    else:
        r = HTMLSession().get(url, headers=header, timeout=2)

    if r.status_code != 200:
        # print('错误代码 %s' % r.status_code)
        raise requests.exceptions.HTTPError('错误代码 %s' % r.status_code)

    if force_render:
        r.html.render()
    return r


async def async_send_request_get_response(url, if_use_proxy=False, proxies=None,
                                          header=self_const.HEADER,
                                          force_render=False):
    '''
    requests-html的异步模式下，必须返回await asession.get
    :param url: request的地址
    :param if_use_proxy:
    :param proxies: 用来来接待代理网页的代理
    :param header:
    :param force_render: 是否要进行render，默认True。如果是静态页面，无需render
    :return: 无
    '''
    if if_use_proxy:
        r = await AsyncHTMLSession().get(url, headers=header, proxies=proxies,
                                  timeout=5)
    else:
        r = await AsyncHTMLSession().get(url, headers=header, timeout=2)

    if force_render:
        await r.html.arender()
    return r


def download_file(url, save_path, headers=self_const.HEADER, proxies=None):
    '''
    下载文件
    :param url:
    :param save_path:
    :return: Error（下载失败）；None（下载成功）
    '''
    try:
        if proxies is not None:
            r = requests.get(url, proxies=proxies, headers=self_const.HEADER)
        else:
            r = requests.get(url, headers=self_const.HEADER)
        with open(save_path, "wb") as code:
            code.write(r.content)
    except Exception as e:
        raise e

    return None


def download_unzip_firefox_driver(python_dir, browser_ver, os_type,os_bits,proxies=None,headers=None):
    '''
    下载浏览器对应的driver到python目录下，然后解压
    :param python_dir:
    :param browser_ver: str，通过get_firefox_version或者get_chrome_version获得
    :param os_type:enum，Win或者Linux
    :param os_bits：enum，32或64bit
    :param proxies:下载文件需要的代理
    :param headers：下载文件需要的header
    :return:
    '''

    # firefox的版本必须大于60
    if int(browser_ver.split('.')[0]) < 60:
        raise EnvironmentError("当前Firefox版本过低，请升级到60或者以上版本")
    # 根据os_type和os_bits确定url
    url_dict ={
        'WindowsWin32':'https://github.com/mozilla/geckodriver/releases/download/v0.26.0/geckodriver-v0.26.0-win32.zip',
        'WindowsWin64': 'https://github.com/mozilla/geckodriver/releases/download/v0.26.0/geckodriver-v0.26.0-win64.zip',
    }
    if os_type == self_enum.OsType.Linux:
        raise EnvironmentError("当前尚不支持Linux")

    url = url_dict[os_type.name + os_bits.name]
    file_name = url.split('/')[-1]
    save_path = os.path.join(python_dir, file_name)
    if os.path.exists(save_path) and self_software.is_valida_zip_file(save_path):
        self_software.unzip_file(file_path=save_path, save_path=python_dir)
        return
    # print('开始时下载')
    # 检测是否需要代理
    if detect_if_need_proxy(url):
        if proxies is None:
            raise ValueError("需要使用代理，但是未设置代理")
        # 检测代理是否可用
        base_url = extract_base_url_from_url(url)
        if not detect_if_proxy_usable(proxies=proxies, timeout=5, url=base_url):
            raise ValueError("设置的代理无法连接%s" % base_url)
        # 代理下载
        download_file(url=url, save_path=save_path, proxies=proxies,headers=headers)
    else:
        # 非代理下载
        download_file(url=url, save_path=save_path)

    self_software.unzip_file(file_path=save_path, save_path=python_dir)


def download_unzip_chrome_driver(python_dir, browser_ver, os_type,os_bits,proxies=None,headers=None):
    '''
    下载浏览器对应的driver到python目录下，然后解压
    :param python_dir:
    :param browser_ver: str，通过get_firefox_version或者get_chrome_version获得
    :param os_type:enum，Win或者Linux
    :param os_bits：enum，32或64bit
    :param proxies:下载文件需要的代理
    :param headers：下载文件需要的header
    :return:
    '''

    # chrome的版本必须大于70
    if int(browser_ver.split('.')[0]) < 70:
        raise EnvironmentError("当前Chrome版本过低，请升级到70或者以上版本")

    if os_type == self_enum.OsType.Linux:
        raise EnvironmentError("当前尚不支持Linux")

    url = 'https://npm.taobao.org/mirrors/chromedriver/%s/chromedriver_win32.zip' % browser_ver
    file_name = url.split('/')[-1]
    save_path = os.path.join(python_dir, file_name)
    if os.path.exists(save_path) and self_software.is_valida_zip_file(save_path):
        self_software.unzip_file(file_path=save_path, save_path=python_dir)
        return
    # print(url)
    # 检测是否需要代理
    if detect_if_need_proxy(url):
        if proxies is None:
            raise ValueError("需要使用代理，但是未设置代理")
        # 检测代理是否可用
        base_url = extract_base_url_from_url(url)
        if not detect_if_proxy_usable(proxies=proxies, timeout=5, url=base_url):
            raise ValueError("设置的代理无法连接%s" % base_url)
        # 代理下载
        if not detect_url_exist(url=url,proxies=proxies,headers=headers):
            raise ValueError("指定的下载链接%s不存在，可能是chrome的版本不支持" % url)
        download_file(url=url, save_path=save_path, proxies=proxies,headers=headers)
    else:
        # 非代理下载
        if not detect_url_exist(url=url,proxies=proxies,headers=headers):
            raise ValueError("指定的下载链接%s不存在，可能是chrome的版本不支持" % url)
        download_file(url=url, save_path=save_path)

    self_software.unzip_file(file_path=save_path, save_path=python_dir)


if __name__ == '__main__':
    pass
    # proxies= gen_proxies_from_ip('87.254.212.121:8080')
    # # download_file(url='http://selenium-release.storage.googleapis.com/3.141/IEDriverServer_x64_3.141.5.zip',
    # #               save_path='d:\\IEDriverServer_x64_3.141.5.zip',proxies=proxies)
    #
    # python_dir=self_software.check_minimum_python_version()
    # chrome_ver = self_software.get_chrome_version()
    # download_unzip_chrome_driver(python_dir=python_dir,  browser_ver=chrome_ver,
    #                       os_type=self_enum.OsType.Windows,os_bits=self_enum.WindowsBits.Win64, proxies=proxies)


# -*- coding:utf-8 -*-
'''
获取软件信息/安装卸载软件
check_minimum_python_version(ver=str): Error（python版本不匹配或者未安装）；python安装路径
check_firefox_version(): None（未安装FF）；FF版本（自动补齐.0）
check_chrome_version(): None（未安装）：chrome版本
check_driver_exist(python_dir, browser_type): 检查对应的driver在python目录下是否存在
unzip_file():解压zip文件到指定目录
is_valid_zip_file():是否为合格的zip文件
'''

import sys
import os
import winreg
import urllib3
import zipfile
import zhangwei_helper.enum.SelfEnum as zw_enum

def check_minimum_python_version(ver=None):
    '''
    :param ver:None或者字符，例如'3.6'
    :return: error或者python安装路径
    '''
    if ver is not None:
        # check the expected version is correct
        tmp = ver.split('.')
        expected_major_ver = int(tmp[0])
        expected_minor_ver = int(tmp[1])
        expected_micro_ver = 0
        if len(tmp) == 3:
            expected_micro_ver = int(tmp[2])

    tmp = sys.version_info
    exist_major_ver = tmp.major
    exist_minor_ver = tmp.minor
    exist_micro_ver = tmp.micro
    exist_ver = '%s.%s.%s' % (tmp.major, tmp.minor, tmp.micro)

    python_ver_match = None
    if ver is not None:
        python_ver_match = False
        if exist_major_ver == expected_major_ver:
            if exist_minor_ver > expected_minor_ver:
                python_ver_match = True
            elif exist_minor_ver == expected_minor_ver:
                if exist_micro_ver >= expected_micro_ver:
                    python_ver_match = True

    if python_ver_match is not None:
        if not python_ver_match:
            raise EnvironmentError('当前python版本' + exist_ver + '不匹配期望的版本' + ver + ',请重新安装')


    try:
        # HKEY_CURRENT_USER\Software\Python\PythonCore\3.8\InstallPath
        sub_item = r'Software\Python\PythonCore\%s.%s\InstallPath' % (exist_major_ver, exist_minor_ver)
        python_item = winreg.OpenKey(winreg.HKEY_CURRENT_USER, sub_item)
    except Exception as e:
        raise EnvironmentError('无法找到python安装路径，请重新安装python%s或者以上版本' % ver)
        # return None

    val, t = winreg.QueryValueEx(python_item, 'ExecutablePath')
    return os.path.dirname(val)


def get_firefox_version():
    '''

    :return:返回最新的firefox版本
    '''
    # HKEY_LOCAL_MACHINE\SOFTWARE\Mozilla\Mozilla Firefox
    try:
        ff_item = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, r'SOFTWARE\Mozilla')
    except Exception as e:
        print('尚未安装firefox')
        return None

    ff_sub_item = winreg.OpenKey(ff_item, r'Mozilla Firefox')
    val, t = winreg.QueryValueEx(ff_sub_item, r'CurrentVersion')
    ff_ver = val.split(' ')[0]
    return ff_ver


def get_chrome_version():
    '''

    :return:None(未安装)；返回最新的chrome版本
    '''
    # HKEY_CURRENT_USER\Software\Google\Chrome\BLBeacon
    # HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Uninstall\Google Chrome
    try:
        chrome_item = winreg.OpenKey(winreg.HKEY_CURRENT_USER,
                                     r'Software\Microsoft\Windows\CurrentVersion\Uninstall\Google Chrome')
    except Exception as e:
        # print(e)
        print('尚未安装chrome')
        return None

    val, t = winreg.QueryValueEx(chrome_item, r'version')
    return val


def check_driver_exist(python_dir, browser_type):
    '''
    检测python_dir下，浏览器browser_type对应的driver是否已经安装
    :param python_dir: python安装路径，将driver安装在其下
    :param browser_type: 只支持ff和chrome
    :return: boolean。 driver是否存在
    '''
    if browser_type == zw_enum.BrowserType.FireFox:
        file_name = 'geckodriver.exe'
    if browser_type == zw_enum.BrowserType.Chrome:
        file_name = 'chromedriver.exe'

    file_path = os.path.join(python_dir, file_name)
    return os.path.exists(file_path)


def unzip_file(file_path, save_path):
    '''

    :param file_path:待解压的文件
    :param save_path: 解压到的目录
    :return:
    '''
    # 解压zip文件
    f = zipfile.ZipFile(file_path, 'r')
    for file in f.namelist():
        f.extract(file, save_path)
    f.close()

def is_valida_zip_file(file_path):
    return zipfile.is_zipfile(file_path)

if __name__ == '__main__':
    # unzip_file(file_path=r'C:\Python38\geckodriver-v0.26.0-win64.zip', save_path=r'C:\Python38')
    print(is_valida_zip_file(r'C:\Python38\geckodriver-v0.26.0-win64.zip'))




#! /usr/bin/env python3
# -*- coding:utf-8 -*-
'''
get_cpu_bits(): 枚举(CpuBits)：cpu的位数
get_os_type(): 枚举(OsType)：os的类型：windows或者linux
get_windows_ver(): 枚举(WindowsVersion)：windows的版本（7/8/10）
get_windows_bits(): 枚举(WindowsBits)：windows的位数：32或者64
windows_login_as_admin(): Boolean：当前是否以admin登录
get_python_major_version():枚举(PythonVersion)：返回python的大版本号：2或者3或者unknown
'''
import sys
import os
import platform
import ctypes
import zhangwei_helper.enum.SelfEnum as self_enum


def get_cpu_bits():
    '''
    CPU的位数，是32bit，还是64bit
    :return:
    '''
    if os.environ.get('PROCESSOR_ARCHITECTURE') == 'AMD64':
        return self_enum.CpuBits.bits64
    else:
        return self_enum.CpuBits.bits32


def get_os_type():
    '''
    操作系统的类型，是win还是linux
    :return:
    '''
    if platform.system() == 'Windows':
        return self_enum.OsType.Windows
    if platform.system() == 'Linux':
        return self_enum.OsType.Linux


def get_windows_ver():
    '''
    windows的版本
    :return:
    '''
    ver = platform.platform().split('-')[1:2:1][0]

    if ver == '7':
        return self_enum.WindowsVersion.Windows7
    elif ver == '8':
        return self_enum.WindowsVersion.Windows8
    elif ver == '10':
        return self_enum.WindowsVersion.Windows10
    else:
        return self_enum.WindowsVersion.Unknown


def get_windows_bits():
    '''
    windows的位数
    :return:
    '''
    if 'PROGRAMFILES(X86)' in os.environ:
        return self_enum.WindowsBits.Win64
    else:
        return self_enum.WindowsBits.Win32


def windows_login_as_admin():
    if get_os_type() == self_enum.OsType.Windows:
        return ctypes.windll.shell32.IsUserAnAdmin() != 0


def get_python_major_version():
    major_version = sys.version_info[0]
    if major_version == 2:
        return self_enum.PythonVersion.Python2
    elif major_version == 3:
        return self_enum.PythonVersion.Python3
    else:
        return self_enum.PythonVersion.Unknown
# print(windows_login_as_admin())


#! /usr/bin/env python3
# -*- coding:utf-8 -*-
'''
CpuBits: bits64/bits32
OsType: Windows/Linux
WindowsVersion: Windows7/Windows8/Windows10/Unknown
WindowsBits: Win32/Win64
PythonVersion: Python2/Python3/Unknown
ProxyType: Transparent/Anonymous/High_anonymous
BrowserType: FireFox/Chrome/All
'''
from enum import Enum, unique


@unique
class CpuBits(Enum):
    bits64 = 0
    bits32 = 1


@unique
class OsType(Enum):
    Windows = 0
    Linux = 1


@unique
class WindowsVersion(Enum):
    Windows7 = 0
    Windows8 = 1
    Windows10 = 2
    Unknown = 3


@unique
class WindowsBits(Enum):
    Win32 = 0
    Win64 = 1

@unique
class PythonVersion(Enum):
    Python2 = 0
    Python3 = 1
    Unknown = 2

@unique
class ProxyType(Enum):
    # 透明：对方服务器知道你使用了代理，也知道你的真实IP。
    # REMOTE_ADDR = ProxyIP，HTTP_VIA = ProxyIP，HTTP_X_FORWARDED_FOR = YourIP
    Transparent = 0
    # 匿名：对方服务器知道你使用了代理，但不知道你的真实IP。
    # REMOTE_ADDR = ProxyIP，HTTP_VIA = ProxyIP，HTTP_X_FORWARDED_FOR = ProxyIP
    Anonymous = 1
    # 高匿名：对方服务器不知道你使用了代理，也不知道你的真实IP。
    # REMOTE_ADDR = ProxyIP，HTTP_VIA = NULL，HTTP_X_FORWARDED_FOR = NULL
    High_anonymous = 2
    All = 3



@unique
class BrowserType(Enum):
    FireFox = 0
    Chrome = 1
    All = 2


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="zhangwei_helper",
    # username
    version="0.0.2",
    author="zwzw911",
    author_email="zwzw911110@163.com",
    description="A package include all common part,like function,enum,written \
                 by zhangwei(zwzw911)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/zwzw911/zhangwei_helper",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows",
    ],
    python_requires='>=3.6',
)


# -*- coding:utf-8 -*-

# 如果只是用来判断是否需要使用代理，则无需随机生成header，使用固定的header即可
HEADER = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:74.0) Gecko/20100101 Firefox/74.0',
    'Accept': 'text/html, application/xhtml+xml, application/xml;q = 0.9, image/webp, image/apng, */*;q = 0.8, application/signed-exchange;v = b3',
    # 'Accept-Encoding': 'gzip, deflate, br',
    'Accept-Encoding': '*',  # 某些网站，及时可以使用br，也需要设置成*，否则返回乱码
    'Accept-Language': 'zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2',
    'Connection': 'keep-alive'}